@@ -151,6 +151,94 @@ class SoftLinx:
                 self.protocolSteps.append(step)
         return step
 
+    def parallel(
+        self,
+        branches=[[]],
+        isActive=True,
+        displayName="Parallel Activity",
+        orderByVariable=False,
+        orderVariable="",
+        fixedOrder=[],
+        index=None,
+        inplace=True,
+    ):
+        if not isinstance(branches, list) or not all(
+            [isinstance(branch, list) for branch in branches]
+        ):
+            raise ValueError("branches must be a list of lists.")
+        step = {
+            "type": "ParallelActivity",
+            "DisplayName": str(displayName),
+            "Command": "ParallelActivity",
+            "ToolTip": "",
+            "SLXId": "0ed9f8ba-5f2b-4f1c-a33a-edaec93b23c",
+            "isActive": str(isActive),
+            "branches": branches,
+            "branchOrder": str(fixedOrder),
+            "orderVariable": orderVariable,
+        }
+        if not orderByVariable and fixedOrder == []:
+            step["IsStandard"] = "True"
+        else:
+            step["IsStandard"] = "False"
+        if orderByVariable:
+            step["IsByVariable"] = "True"
+        else:
+            step["IsByVariable"] = "False"
+        if fixedOrder != []:
+            step["IsOrdered"] = "True"
+        else:
+            step["IsOrdered"] = "False"
+        if inplace:
+            if index != None:
+                self.protocolSteps.insert(index, step)
+            else:
+                self.protocolSteps.append(step)
+        return step
+
+    def runProgram(
+        self,
+        command="",
+        variable_name="",
+        hide_prompt=False,
+        is_variable=False,
+        is_constant=True,
+        wait_for_complete=False,
+        arguments="",
+        isActive=True,
+        index=None,
+        inplace=True,
+    ):
+        if not isinstance(hide_prompt, bool):
+            raise TypeError("hide_prompt should be a boolean.")
+        if not isinstance(is_variable, bool):
+            raise TypeError("is_variable should be a boolean.")
+        if not isinstance(is_constant, bool):
+            raise TypeError("is_constant should be a boolean.")
+        if not isinstance(wait_for_complete, bool):
+            raise TypeError("wait_for_complete should be a boolean.")
+        step = {
+            "type": "RunProgramActivity",
+            "CommandLine": "",
+            "Description": "Run: '%s'" % str(command),
+            "Command": str(command),
+            "VariableName": str(variable_name),
+            "HidePrompt": bool(hide_prompt),
+            "IsVariable": bool(is_variable),
+            "IsConstant": bool(is_constant),
+            "Arguments": str(arguments),
+            "WaitForComplete": bool(wait_for_complete),
+            "SLXId": "aa9c6603-467b-4c4f-81b3-49bb7dd0f768",
+            "ToolTip": "Run: '%s'" % str(command),
+            "isActive": str(isActive),
+        }
+        if inplace:
+            if index != None:
+                self.protocolSteps.insert(index, step)
+            else:
+                self.protocolSteps.append(step)
+        return step
+
     # * Plate Crane Steps * #
     def plateCraneMoveCrane(
         self,
@@ -274,6 +362,110 @@ class SoftLinx:
                 self.protocolSteps.append(step)
         return step
 
+    def plateCraneRemoveLid(
+        self,
+        positionsFrom=["SoftLinx.PlateCrane.Stack1"],
+        positionsTo=["SoftLinx.PlateCrane.Stack2"],
+        onEmptyStack="End Method",
+        moveUpAndHoldLid=False,
+        moveUpAmount=10,
+        checkForPlatesInAllStacks=False,
+        isActive=True,
+        index=None,
+        inplace=True,
+    ):
+        if not isinstance(positionsFrom, list) or not isinstance(positionsTo, list):
+            raise ValueError("positionsFrom and positionsTo must be Lists.")
+        onEmptyStackDict = {
+            "End Method": 0,
+            "Post Message and Continue": 1,
+        }
+        if isinstance(onEmptyStack, str):
+            try:
+                onEmptyStackVal = onEmptyStackDict[onEmptyStack]
+            except:
+                raise ValueError(
+                    "onEmptyStack must be one of the following:"
+                    + str([key + ", " for key in onEmptyStackDict])
+                )
+        else:
+            raise ValueError(
+                "onEmptyStack must be one of the following:"
+                + str([key + ", " for key in onEmptyStackDict])
+            )
+        if checkForPlatesInAllStacks:
+            onEmptyStackVal += 240
+        if moveUpAndHoldLid:
+            if not isinstance(moveUpAmount, int):
+                raise ValueError("moveUp must be an integer")
+        else:
+            moveUpAmount = 0
+
+        step = {
+            "type": "RemoveLid",
+            "Command": "Remove Lid",
+            "Description": "RemoveLid from: %s" % ",".join(positionsFrom)
+            + " to %s" % ",".join(positionsTo),
+            "SLXId": "99bd7bce-806b-416b-9458-c39afda998d8",
+            "ToolTip": "RemoveLid from: %s" % ",".join(positionsFrom)
+            + " to %s" % ",".join(positionsTo),
+            "isActive": str(isActive),
+            "system": "PlateCrane",
+            "args": [
+                ["x:String", ",".join(positionsFrom)],
+                ["x:String", ",".join(positionsTo)],
+                ["x:String", str(moveUpAmount)],
+                ["x:String", str(onEmptyStackVal)],
+                ["x:String", " "],
+            ],
+        }
+
+        if inplace:
+            if index != None:
+                self.protocolSteps.insert(index, step)
+            else:
+                self.protocolSteps.append(step)
+        return step
+
+    def plateCraneReplaceLid(
+        self,
+        positionsFrom=["SoftLinx.PlateCrane.Stack1"],
+        positionsTo=["SoftLinx.PlateCrane.Stack2"],
+        lidInGripper=False,
+        isActive=True,
+        index=None,
+        inplace=True,
+    ):
+        if lidInGripper:
+            positionsFrom = [" "]
+        if not isinstance(positionsFrom, list) or not isinstance(positionsTo, list):
+            raise ValueError("positionsFrom and positionsTo must be Lists.")
+
+        step = {
+            "type": "ReplaceLid",
+            "Command": "Replace Lid",
+            "Description": "ReplaceLid from: %s" % ",".join(positionsFrom)
+            + " onto Plate in %s" % ",".join(positionsTo),
+            "SLXId": "99bd7bce-806b-416b-9458-c39afda998d8",
+            "ToolTip": "ReplaceLid from: %s" % ",".join(positionsFrom)
+            + " onto Plate in %s" % ",".join(positionsTo),
+            "isActive": str(isActive),
+            "system": "PlateCrane",
+            "args": [
+                ["x:String", ",".join(positionsFrom)],
+                ["x:String", ",".join(positionsTo)],
+                ["x:String", "False"],
+                ["x:String", " "],
+            ],
+        }
+
+        if inplace:
+            if index != None:
+                self.protocolSteps.insert(index, step)
+            else:
+                self.protocolSteps.append(step)
+        return step
+
     # * SoloSoft Steps * #
     def soloSoftRun(
         self, filename="protocol.hso", isActive=True, index=None, inplace=True
@@ -580,27 +772,67 @@ class SoftLinx:
                 "Cannot generate AutoHotKey script without a softlinx filename."
             )
         with open(ahk_filename, "w") as file:
-            file.write("#SingleInstance, Force\n")
-            file.write("SendMode Input\n")
-            file.write("SetWorkingDir, %A_ScriptDir%\n")
-            file.write("\n")
             file.write(
-                'Run, "C:\Program Files (x86)\Hudson Robotics\SoftLinx V\SoftLinxVProtocolEditor.exe" %A_ScriptDir%\\'
-                + softlinx_filename
-                + "\n"
+                """
+#SingleInstance, Force
+#WinActivateForce
+SendMode Input
+SetWorkingDir, %%A_ScriptDir%%
+SetTitleMatchMode, 2
+
+if WinExist("User Cancelled Run")
+{
+    WinActivate
+    Send, {Enter}
+    Sleep, 500
+}
+if WinExist("ahk_exe SoftLinxVProtocolEditor.exe")
+{
+    WinClose
+    Sleep, 1000
+    if WinExist("ahk_exe SoftLinxVProtocolEditor.exe")
+    {
+        MsgBox, Couldn't close SoftLinx V, please close it manually and restart this run.
+        return
+    }
+}
+if WinExist("ahk_exe SOLOSoft.exe")
+{
+    MsgBox, SOLOSoft is still running. Please kill it, then press "OK" below to resume execution.
+    if WinExist("ahk_exe SOLOSoft.exe")
+    {
+        MsgBox, SOLOSoft is still running, please close it manually and restart this run.
+        return
+    }
+}
+Sleep, 1000
+Run, "C:\Program Files (x86)\Hudson Robotics\SoftLinx V\SoftLinxVProtocolEditor.exe" %%A_ScriptDir%%\\%s
+WinWaitActive, SoftLinx V,,10
+if ErrorLevel
+{
+    WinGetActiveTitle, Title
+    MsgBox, Couldn't find SoftLinx V window. The active window is "%%Title%%".
+    return
+}
+Sleep, 5000
+MouseClick, Left, 300, 45
+Sleep, 1000
+if WinActive("Not Saved") {
+    Send, {Tab}{Enter}
+    Sleep, 5000
+}
+if WinActive("Start Now?") {
+    Send, {Enter}
+    Sleep, 5000
+}
+else
+{
+    MsgBox, Error starting run.
+    return
+}
+                """
+                % softlinx_filename
             )
-            file.write("Sleep, 5000\n")
-            file.write("WinActivate, SoftLinx V\n")
-            file.write("MouseClick, Left, 300, 45\n")
-            file.write("Sleep, 1000\n")
-            file.write('if WinActive("Not Saved") {\n')
-            file.write("\tSend, {Tab}{Enter}\n")
-            file.write("\tSleep, 5000\n")
-            file.write("}\n")
-            file.write('if WinActive("Start Now?") {\n')
-            file.write("\tSend, {Enter}\n")
-            file.write("\tSleep, 5000\n")
-            file.write("}\n")
 
     def generatePluginVariables(self, parentXML, system):
         variable = ET.SubElement(
@@ -713,10 +945,168 @@ class SoftLinx:
             },
         )
 
+    def generateParallelXML(self, parent, step):
+        activity_dict = {
+            "Text": "{x:Null}",
+            "ActivityLabel": "",
+            "DisplayName": step["DisplayName"],
+            "HasConstraints": "False",
+            "SLXId": step["SLXId"],
+            "ToolTip": step["ToolTip"],
+            "UserComments": "",
+            "isActive": step["isActive"],
+            "isSetup": "True",
+        }
+        parallel_xml = ET.SubElement(parent, "ParallelActivity", activity_dict)
+        parallelactivity_activities = ET.SubElement(
+            parallel_xml, "ParallelActivity.Activities"
+        )
+        scg_list = ET.SubElement(
+            parallelactivity_activities,
+            "scg:List",
+            {
+                "x:TypeArguments": "p:Activity",
+                "Capacity": "4",
+            },
+        )
+        branch_count = 0
+        for branch in step["branches"]:
+            branch_count += 1
+            branch_activity = ET.SubElement(
+                scg_list,
+                "BranchActivity",
+                {
+                    "ActivityLabel": "",
+                    "DisplayName": "Branch " + str(branch_count),
+                    "HasConstraints": "False",
+                    "SLXId": "38784045-5053-4b14-bbc1-cf98a8b35a23",
+                    "ToolTip": "",
+                    "UserComments": "",
+                    "isActive": "True",
+                    "isSetup": "True",
+                },
+            )
+            branch_activity_activities = ET.SubElement(
+                branch_activity, "BranchActivity.Activities"
+            )
+            branch_scg_list = ET.SubElement(
+                branch_activity_activities,
+                "scg:List",
+                {
+                    "x:TypeArguments": "p:Activity",
+                    "Capacity": "4",
+                },
+            )
+            for substep in branch:
+                self.generateStepXML(branch_scg_list, substep)
+            branch_activity_activities2 = ET.SubElement(
+                branch_activity, "BranchActivity.Activities2"
+            )
+            branch_scg_list2 = ET.SubElement(
+                branch_activity_activities2,
+                "scg:List",
+                {
+                    "x:TypeArguments": "p:Activity",
+                    "Capacity": "0",
+                },
+            )
+            branch_time_constraints = ET.SubElement(
+                branch_activity, "BranchActivity.TimeConstraints"
+            )
+            time_scg_list = ET.SubElement(
+                branch_time_constraints,
+                "scg:List",
+                {
+                    "x:TypeArguments": "hwab:TimeConstraint",
+                    "Capacity": "0",
+                },
+            )
+
+        parallel_activities2 = ET.SubElement(
+            parallel_xml, "ParallelActivity.Activities2"
+        )
+        scg_list = ET.SubElement(
+            parallel_activities2,
+            "scg:List",
+            {
+                "x:TypeArguments": "p:Activity",
+                "Capacity": "0",
+            },
+        )
+        parallel_arguments = ET.SubElement(parallel_xml, "ParallelActivity.Arguments")
+        ET.SubElement(
+            parallel_arguments,
+            "ParallelActivityArguments",
+            {
+                "BranchOrder": step["branchOrder"],
+                "IsByVariable": step["IsByVariable"],
+                "IsOrdered": step["IsOrdered"],
+                "IsStandard": step["IsStandard"],
+                "VariableName": step["orderVariable"],
+            },
+        )
+        parallel_timeconstraints = ET.SubElement(
+            parallel_xml, "ParallelActivity.TimeConstraints"
+        )
+        ET.SubElement(
+            parallel_timeconstraints,
+            "scg:List",
+            {
+                "x:TypeArguments": "hwab:TimeConstraint",
+                "Capacity": "0",
+            },
+        )
+
+    def generateRunProgramXML(self, parent, step):
+        activity_dict = {
+            "CommandLine": step["CommandLine"],
+            "Description": step["Description"],
+            "DisplayName": "Run Program",
+            "HasConstraints": "False",
+            "SLXId": step["SLXId"],
+            "ToolTip": step["ToolTip"],
+            "UserComments": "",
+            "isActive": step["isActive"],
+            "isCanceled": "False",
+            "isSetup": "True",
+        }
+        run_xml = ET.SubElement(parent, "RunProgramActivity", activity_dict)
+        run_arguments = ET.SubElement(run_xml, "RunProgramActivity.Arguments")
+        ET.SubElement(
+            run_arguments,
+            "RunProgramActivityArguments",
+            {
+                "Command": str(step["Command"]),
+                "VariableName": str(step["VariableName"]),
+                "HidePrompt": str(step["HidePrompt"]),
+                "IsVariable": str(step["IsVariable"]),
+                "IsConstant": str(step["IsConstant"]),
+                "Arguments": str(step["Arguments"]),
+                "WaitForComplete": str(step["WaitForComplete"]),
+            },
+        )
+        run_timeconstraints = ET.SubElement(
+            run_xml, "RunProgramActivity.TimeConstraints"
+        )
+        ET.SubElement(
+            run_timeconstraints,
+            "scg:List",
+            {
+                "x:TypeArguments": "hwab:TimeConstraint",
+                "Capacity": "0",
+            },
+        )
+
     def generateStepXML(self, scg_list, step):
         if step["type"] == "IfElseActivity":
             self.generateConditionalXML(scg_list, step)
             return
+        if step["type"] == "IfElseActivity":
+            self.generateParallelXML(scg_list, step)
+            return
+        if step["type"] == "RunProgramActivity":
+            self.generateRunProgramXML(scg_list, step)
+            return
         self.plugin_flags[step["system"]] = True
         activity_dict = {
             "IconPath": "{x:Null}",
